import * as d3Select from 'd3-selection'
import * as d3Shape from 'd3-shape'
import * as d3Interpolate from 'd3-interpolate'
import * as d3Array from 'd3-array'
import * as d3Ease from 'd3-ease'
import { color } from '../../../config'
import { getElem } from '../../../util/getElem'

const d3 = Object.assign(
  {},
  d3Select,
  d3Shape,
  d3Interpolate,
  d3Array,
  d3Ease,
)

interface IchartOption {
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  data: any
  radius: any
  roseType: boolean
  gname: string
  piename: string
  duration: number
  tooltip: any
}

let oldData: any = []
let currentRadius: any = null

/**
 * generate and animation
 * @param {IchartOption} chartOption
 * @return {null}
 */
const chart = (chartOption: IchartOption) => {
  const {
    chartWidth,
    chartHeight,
    paddingLeft,
    paddingTop,
    svg,
    data,
    radius,
    roseType,
    gname,
    piename,
    duration,
    tooltip,
  } = chartOption

  const pie = d3.pie()
    .sort(null)
    .value((d: any) => d.value)

  const min = d3.min([chartWidth, chartHeight])
  const half = min / 2

  currentRadius = radius

  // 各类型半径
  const arcTypes = {
    pie: () => {
      return {
        innerRadius: 0,
        outerRadius: half * parseInt(radius, 10) / 100,
      }
    },
    circle: () => {
      const result = currentRadius.map((item: string) => parseInt(item, 10))
      return {
        innerRadius: half * result[0] / 100,
        outerRadius: half * result[1] / 100,
      }
    },
  }

  const Radius = arcTypes[radiusType(radius)]()

  const arc = d3.arc()
    .innerRadius(Radius.innerRadius)
    .outerRadius(Radius.outerRadius)

  // 图 gp
  const gp = getElem(svg, 'g', gname)
  gp.attr('transform', `translate(${paddingLeft}, ${paddingTop})`)

  const update = gp.selectAll(`.${piename}`).data(pie(data))
  const enter = update.enter()
  const exit = update.exit()

  enter.append('path')
    .attr('class', piename)
    .attr('fill', (d: any, i: number) => color[i])
    .style('cursor', 'pointer')
    .attr('transform', `translate(${chartWidth / 2}, ${chartHeight / 2})`)
    .on('mouseenter', function(d: any, i: any) {
      // radius 和 currentRadius 值不等
      const newRadius = arcTypes[radiusType(currentRadius)]()
      d3.select(this)
        .transition()
        .ease(d3.easeBounceOut)
        .duration(300)
        .attrTween('d', (d: any) => {
          const r = d3.interpolate(newRadius.outerRadius, newRadius.outerRadius + 20)
          const z = pie(oldData)[i]
          return (t: number) => {
            const Arc = d3.arc()
              .innerRadius(newRadius.innerRadius)
              .outerRadius(r(t))
            return Arc(z)
          }
        })
      tooltip.changeIndex(i)
    })
    .on('mouseleave', function(d: any, i: any) {
      const newRadius = arcTypes[radiusType(currentRadius)]()
      d3.select(this)
        .transition()
        .ease(d3.easeBounceOut)
        .duration(300)
        .attrTween('d', (d: any) => {
          const r = d3.interpolate(newRadius.outerRadius + 20, newRadius.outerRadius)
          const z = pie(oldData)[i]
          return (t: number) => {
            const Arc = d3.arc()
              .innerRadius(newRadius.innerRadius)
              .outerRadius(r(t))
            return Arc(z)
          }
        })
      tooltip.changeIndex(null)
    })
    .transition()
    .duration(duration)
    .attrTween('d', (d: any) => {
      const i = d3.interpolate({ startAngle: 0, endAngle: 0 }, d) // 弧度差值
      return (t: number) => {
        if (t === 1) {
          oldData = data
        }
        return arc(i(t))
      }
    })

  update
    .transition()
    .duration(duration)
    .attrTween('d', (d: any, i: number) => {
      const j = d3.interpolate(pie(oldData)[i], d)
      return (t: number) => {
        if (t === 1) {
          oldData = data
        }
        return arc(j(t))
      }
    })

  exit
    .transition()
    .duration(duration)
    .attrTween('d', (d: any, i: number) => {
      const k = d3.interpolate(pie(oldData)[i],
        { startAngle: pie(oldData)[i].endAngle, endAngle: pie(oldData)[i].endAngle })
      return (t: number) => {
        return arc(k(t))
      }
    })
    .remove()

}

const radiusType = (radius: any) => {
  return typeof radius === 'string' ? 'pie' : 'circle'
}

export default chart
