import * as d3Select from 'd3-selection'
import * as d3Ease from 'd3-ease'
import * as _ from 'lodash'
import { getElem } from '../util/getElem'
import { color } from '../config'

const d3 = Object.assign(
  {},
  d3Select,
  d3Ease,
)

interface Itooltip {
  elem: any
  width: number
  height: number
  series: any
  xAxis: any
}

/**
 * generate tooltip HTMLDivElement
 * @param {Itooltip} tooltipOption
 * @return tooltip
 */
class Tooltip {
  currentIndex: number
  offsetWidth: number
  offsetHeight: number
  series: any[]
  xAxis: any[]
  width: number
  height: number
  showList: any

  constructor() {
    this.offsetWidth = 20 // tooltip 与 mouse 偏移量 x
    this.offsetHeight = 20 // tooltip 与 mouse 偏移量 y
    this.currentIndex = null // 当前所在元素 index
    this.series = [] // 数据
    this.xAxis = [] // xAxis
    this.width = 0 // elem width
    this.height = 0 // elem height
  }

  init(tooltipOption: Itooltip, showList?: any) {
    const {
      elem,
      width,
      height,
      series,
      xAxis,
    } = tooltipOption

    const tooltip = getElem(elem, 'div', 'yo-tooltip')
    tooltip
      .style('position', 'absolute')
      .style('visibility', 'hidden')
      .style('left', `${width / 2}px`) // tooltip 起始位置居中
      .style('top', `${height / 2}px`)
      .style('border-radius', '6px')
      .style('padding', '5px')
      .style('background', 'rgba(50, 50, 50, .7)')

    this.series = series
    this.xAxis = xAxis
    this.width = width
    this.height = height
    this.showList = showList

    this.tooltipContent(tooltip, elem, showList)
  }

  // 生成 tooltip 内容
  tooltipContent(tooltip: any, elem: any, showList: any) {
    if (tooltip.select('.yo-tooltip-label').empty()) {
      tooltip
        .append('text')
        .attr('class', 'yo-tooltip-label')
        .style('color', '#fff')
        .style('font-size', '14px')
        .text('')

      /**
       * 添加鼠标事件, 只添加一次;
       * 事件绑定在 elem 元素上;
       * 注意: 应该是生成元素后替换文本内容. 由于是 d3.select 方式, 可以先添加事件
       */
      this.addEvents(tooltip, elem)
    }

    const update = tooltip.selectAll('.yo-tooltip-text').data(this.series)
    const enter = update.enter()
    const exit = update.exit()

    const g = enter.append('g')
      .attr('class', 'yo-tooltip-text')
      .style('display', (d: any, i: number) => (!showList || showList[d.name] || !d.name) && 'block' || 'none')

    g.append('span')
      .style('display', 'inline-block')
      .style('margin-right', '5px')
      .style('border-radius', '10px')
      .style('width', '10px')
      .style('height', '10px')
      .style('background-color', (d: any, i: number) => color[i])
    g.append('text')
      .attr('class', (d: any, i: number) => `yo-tooltip-text${i}`)
      .style('color', '#fff')
      .style('font-size', '14px')
      .text('')

    update
      .style('display', (d: any, i: number) => (!showList || showList[d.name] || !d.name) && 'block' || 'none')

    update.select('span')
      .style('background-color', (d: any, i: number) => color[i])

    exit.remove()
  }

  // tooltip 事件
  addEvents(tooltip: any, elem: any) {
    let mouseX = null // 鼠标位置 x
    let mouseY = null // 鼠标位置 y
    const offsetWidth = this.offsetWidth
    const offsetHeight = this.offsetHeight

    const self = this
    elem.on('mousemove', _.throttle(function() {
      const node = tooltip.node().getBoundingClientRect()
      const tooltipWidth = node.width // tooltip 宽度
      const tooltipHeight = node.height // tooltip 高度

      mouseX = d3.mouse(this)[0] + offsetWidth
      mouseY = d3.mouse(this)[1] + offsetHeight

      const index =  self.currentIndex

      // 替换 tooltip 内容
      self.series.map((item: any, key: number) => {
        if (item.data[index]) {
          if (self.xAxis) {
            tooltip.select('.yo-tooltip-label').text(self.xAxis[0].data[index])
            tooltip.select(`.yo-tooltip-text${key}`).text(`${item.name}: ${item.data[index]}`)
          } else {
            // 饼图数据 过滤掉数据后计算百分比
            const data = item.data.filter((item: any) => {
              return !self.showList && true || self.showList[item.name]
            })
            const sum = data.reduce((v1: any, v2: any) => {
              return (v1 + v2.value)
            }, 0)
            const percent = `${(item.data[index].value / sum * 100).toFixed(2)}%`
            tooltip.select('.yo-tooltip-label').text(item.data[index].name)
            tooltip.select(`.yo-tooltip-text${key}`).text(`${item.data[index].value}: ${percent}`)
            tooltip.select('span').style('background-color', color[index])
          }
        }
      })

      // 靠近右侧时
      if (self.width - mouseX < tooltipWidth) {
        mouseX = mouseX - tooltipWidth - offsetWidth
      }

      // 靠近底部时
      if (self.height - mouseY < tooltipHeight) {
        mouseY = mouseY - tooltipHeight - offsetHeight
      }

      if (typeof index === 'number') {
        tooltip
          .transition()
          .duration(200)
          .ease(d3.easeCubicOut)
          .style('visibility', 'visible')
          .style('left', `${mouseX}px`)
          .style('top', `${mouseY}px`)
      } else {
        tooltip
          .style('visibility', 'hidden')
      }

    }, 200, { trailing: false, leading: true }))

    elem.on('mouseleave', function() {
      tooltip
        .style('visibility', 'hidden')
    })
  }

  changeIndex(num: number) {
    this.currentIndex = num
  }
}

export default Tooltip
